from imports import *

"""
This class calculates feature importance

Input: 


"""


class shap_pdp():
    def __init__(self):
        super(shap_pdp, self).__init__()
        self.param= None

    def find(self,  df):

        return df


